@@ -246,6 +246,17 @@
 #'   })
 #' )
 #'
+#' # You can also hard-code the call when it's not easy to
+#' # forward it from the caller
+#'  f <- function() {
+#'   abort("my message", call = call("my_function"))
+#' }
+#' g <- function() {
+#'   f()
+#' }
+#' # Shows that the error occured in `my_function()`
+#' try(g())
+#'
 #' }
 #' @export
 abort <- function(message = NULL,
@@ -1469,9 +1480,7 @@ peek_backtrace_on_error <- function() {
     return(opt)
   }
 
-  if (report_in_progress()) {
-    "branch"
-  } else if (is_interactive()) {
+  if (is_interactive()) {
     "reminder"
   } else {
     "full"
