@@ -67,21 +67,31 @@ cnd_message <- function(cnd, ..., inherit = TRUE, prefix = FALSE) {
   }
 
   if (prefix) {
-    msg <- cnd_message_format_prefixed(cnd, ..., parent = FALSE)
+    # Skip child errors that have empty messages and calls
+    while (!length(msg <- cnd_message_format_prefixed(cnd, ..., parent = FALSE))) {
+      parent <- cnd[["parent"]]
+      if (is_condition(parent)) {
+        cnd <- parent
+      } else {
+        break
+      }
+    }
   } else {
     msg <- cnd_message_format(cnd, ...)
   }
 
+  warning <- inherits(cnd, "warning")
+
   # Parent messages are always prefixed
   while (is_condition(cnd <- cnd$parent)) {
-    parent_msg <- cnd_message_format_prefixed(cnd, parent = TRUE)
+    parent_msg <- cnd_message_format_prefixed(cnd, parent = TRUE, warning = warning)
     msg <- paste_line(msg, parent_msg)
   }
 
   backtrace_on_error <- cnd_backtrace_on_error(orig) %||% "none"
   trace_footer <- format_onerror_backtrace(orig, opt = backtrace_on_error)
 
-  c(msg, trace_footer)
+  paste_line(msg, trace_footer)
 }
 cnd_message_lines <- function(cnd, ...) {
   c(
@@ -127,12 +137,12 @@ cnd_message_format <- function(cnd, ..., alert = FALSE) {
 
   cli_format <- switch(
     cnd_type(cnd),
-    error = cli::format_error,
-    warning = cli::format_warning,
-    cli::format_message
+    error = format_error,
+    warning = format_warning,
+    format_message
   )
 
-  cli_format(glue_escape(lines), .envir = emptyenv())
+  cli_format(glue_escape(lines))
 }
 
 local_cli_indent <- function(frame = caller_env()) {
@@ -149,7 +159,7 @@ cnd_header <- function(cnd, ...) {
   if (is_null(cnd[["header"]])) {
     UseMethod("cnd_header")
   } else {
-    exec_cnd_method("header", cnd)
+    exec_cnd_method("header", cnd, ...)
   }
 }
 #' @export
@@ -163,7 +173,7 @@ cnd_body <- function(cnd, ...) {
   if (is_null(cnd[["body"]])) {
     UseMethod("cnd_body")
   } else {
-    exec_cnd_method("body", cnd)
+    exec_cnd_method("body", cnd, ...)
   }
 }
 #' @export
@@ -177,7 +187,7 @@ cnd_footer <- function(cnd, ...) {
   if (is_null(cnd[["footer"]])) {
     UseMethod("cnd_footer")
   } else {
-    exec_cnd_method("footer", cnd)
+    exec_cnd_method("footer", cnd, ...)
   }
 }
 #' @export
@@ -185,14 +195,14 @@ cnd_footer.default <- function(cnd, ...) {
   chr()
 }
 
-exec_cnd_method <- function(name, cnd) {
+exec_cnd_method <- function(name, cnd, ...) {
   method <- cnd[[name]]
 
   if (is_function(method)) {
-    method(cnd)
+    method(cnd, ...)
   } else if (is_bare_formula(method)) {
     method <- as_function(method)
-    method(cnd)
+    method(cnd, ...)
   } else if (is_character(method)) {
     method
   } else {
@@ -208,7 +218,8 @@ exec_cnd_method <- function(name, cnd) {
 cnd_message_format_prefixed <- function(cnd,
                                         ...,
                                         parent = FALSE,
-                                        alert = NULL) {
+                                        alert = NULL,
+                                        warning = FALSE) {
   type <- cnd_type(cnd)
 
   if (is_null(alert)) {
@@ -217,17 +228,27 @@ cnd_message_format_prefixed <- function(cnd,
 
   if (parent) {
     prefix <- sprintf("Caused by %s", type)
+    # FIXME: Enable this by default later on
+    if (use_red_error_prefix()) {
+      prefix <- col_prefix(prefix, type)
+    }
   } else {
-    prefix <- col_yellow(capitalise(type))
+    prefix <- capitalise(type)
+    prefix <- col_prefix(prefix, type)
   }
 
-  call <- format_error_call(cnd$call)
+  evalq({
+    if (is_true(peek_option("rlang:::error_highlight"))) {
+      local_error_highlight()
+    }
+    call <- format_error_call(cnd[["call"]])
+  })
 
   message <- cnd_message_format(cnd, ..., alert = alert)
   message <- strip_trailing_newline(message)
 
-  if (!nzchar(message)) {
-    return(NULL)
+  if (!nzchar(message) && is_null(call)) {
+    return(character())
   }
 
   has_loc <- FALSE
@@ -235,7 +256,7 @@ cnd_message_format_prefixed <- function(cnd,
   if (is_null(call)) {
     prefix <- sprintf("%s:", prefix)
   } else {
-    src_loc <- src_loc(attr(cnd$call, "srcref"))
+    src_loc <- src_loc(attr(cnd[["call"]], "srcref"))
     if (nzchar(src_loc) && peek_call_format_srcref()) {
       prefix <- sprintf("%s in %s at %s:", prefix, call, src_loc)
       has_loc <- TRUE
@@ -243,9 +264,27 @@ cnd_message_format_prefixed <- function(cnd,
       prefix <- sprintf("%s in %s:", prefix, call)
     }
   }
-  prefix <- style_bold(prefix)
+  if (!warning) {
+    prefix <- style_bold(prefix)
+  }
 
-  paste0(prefix, "\n", message)
+  if (nzchar(message)) {
+    paste0(prefix, "\n", message)
+  } else {
+    prefix
+  }
+}
+
+col_prefix <- function(prefix, type) {
+  if (type == "error" && use_red_error_prefix()) {
+    col_red(prefix)
+  } else {
+    col_yellow(prefix)
+  }
+}
+
+use_red_error_prefix <- function() {
+  peek_option("rlang:::use_red_error_prefix") %||% FALSE
 }
 
 peek_call_format_srcref <- function() {
@@ -303,6 +342,22 @@ on_load({
   })
 })
 
+on_load({
+  s3_register("knitr::sew", "rlang_warning", function(x, options, ...) {
+    # Simulate interactive session to prevent full backtrace from
+    # being included in error message
+    local_interactive()
+
+    # Include backtrace footer option in the condition. Processed by
+    # `cnd_message()`.
+    x <- cnd_set_backtrace_on_error(x, peek_backtrace_on_warning_report())
+
+    # The `sew.error()` method calls `as.character()`, which dispatches
+    # to `cnd_message()`
+    NextMethod()
+  })
+})
+
 #' Format bullets for error messages
 #'
 #' @description
@@ -360,41 +415,6 @@ format_error_bullets <- function(x) {
   .rlang_cli_format_fallback(x)
 }
 
-# FIXME: These won't be needed after warnings and messages have been
-# switched to print-time formatting
-rlang_format_warning <- function(x, env = caller_env()) {
-  rlang_format(x, env, format_warning, cli::format_warning)
-}
-rlang_format_message <- function(x, env = caller_env()) {
-  rlang_format(x, env, format_message, cli::format_message)
-}
-rlang_format <- function(x, env, partial_format, cli_format) {
-  if (!can_format(x)) {
-    return(x)
-  }
-
-  use_cli <- use_cli(env)
-  inline <- use_cli[["inline"]]
-  format <- use_cli[["format"]]
-
-  # Full
-  if (inline && format) {
-    return(.rlang_cli_str_restore(cli_format(x, env), x))
-  }
-
-  # Partial
-  if (format) {
-    if (has_cli_format) {
-      return(partial_format(cli_escape(x)))
-    } else {
-      return(.rlang_cli_format_fallback(x))
-    }
-  }
-
-  # None
-  x
-}
-
 # No-op for the empty string, e.g. for `abort("", class = "foo")` and
 # a `conditionMessage.foo()` method. Don't format inputs escaped with `I()`.
 can_format <- function(x) {
