@@ -435,11 +435,7 @@ format.rlang_trace <- function(x,
 
 arg_match_simplify <- function(simplify, call = caller_env()) {
   if (is_null(simplify)) {
-    if (use_tree_display()) {
-      return("none")
-    } else {
-      return("branch")
-    }
+    return("none")
   }
 
   if (is_string(simplify, "collapse")) {
@@ -449,15 +445,11 @@ arg_match_simplify <- function(simplify, call = caller_env()) {
   arg_match0(simplify, c("none", "branch"), error_call = call)
 }
 arg_match_drop <- function(drop) {
-  if (is_null(drop)) {
-    use_tree_display()
-  } else {
-    drop
-  }
+  drop %||% TRUE
 }
 
 deprecate_collapse <- function() {
-  warn_deprecated("`\"collapse\"` is deprecated as of rlang 1.1.0.\nPlease use `\"none\"` instead.")
+  deprecate_warn("`\"collapse\"` is deprecated as of rlang 1.1.0.\nPlease use `\"none\"` instead.")
 }
 
 trace_format <- function(trace,
@@ -561,7 +553,7 @@ style_locs <- function(locs) {
   chr(map_if(locs, nzchar, col_grey))
 }
 zip_chr <- function(xs, ys) {
-  flatten_chr(map2(xs, ys, function(x, y) {
+  list_c(map2(xs, ys, function(x, y) {
     if (nzchar(y)) {
       c(x, y)
     } else {
@@ -867,7 +859,7 @@ src_loc <- function(srcref) {
   }
 
   line <- srcref[[1]]
-  column <- srcref[[5]] - 1L
+  column <- srcref[[5]]
 
   style_hyperlink(
     paste0(file_trim, ":", line, ":", column),
