@@ -1,4 +1,5 @@
 #include <rlang.h>
+#include "internal.h"
 #include "nse-inject.h"
 #include "utils.h"
 
@@ -81,17 +82,19 @@ r_obj* ffi_enquo(r_obj* sym, r_obj* frame) {
 
 // Match ------------------------------------------------------------------
 
+static
 int arg_match(r_obj* arg,
               r_obj* values,
-              r_obj* error_arg,
-              r_obj* error_call) {
+              struct r_lazy error_arg,
+              struct r_lazy error_call,
+              struct r_lazy call) {
   if (r_typeof(values) != R_TYPE_character) {
-    r_abort("`values` must be a character vector.");
+    r_abort_lazy_call(call, "`values` must be a character vector.");
   }
 
   int values_len = r_length(values);
   if (values_len == 0) {
-    r_abort("`values` must have at least one element.");
+    r_abort_lazy_call(call, "`values` must have at least one element.");
   }
 
   switch (r_typeof(arg)) {
@@ -102,7 +105,9 @@ int arg_match(r_obj* arg,
   case R_TYPE_symbol:
     return arg_match1(r_sym_string(arg), values, error_arg, error_call);
   default:
-    r_abort("%s must be a string or character vector.", r_format_error_arg(error_arg));
+    r_abort_lazy_call(error_call,
+                      "%s must be a string or character vector.",
+                      r_format_lazy_error_arg(error_arg));
   }
 
   int arg_len = r_length(arg);
@@ -112,7 +117,7 @@ int arg_match(r_obj* arg,
   }
 
   if (arg_len != values_len) {
-    r_abort("%s must be a string or have the same length as `values`.", r_format_error_arg(error_arg));
+    r_abort_lazy_call(call, "`arg` must be a string or have the same length as `values`.");
   }
 
   r_obj* const* v_values = r_chr_cbegin(values);
@@ -153,8 +158,12 @@ int arg_match(r_obj* arg,
     }
 
     if (!matched) {
-      error_arg = KEEP(wrap_chr(error_arg));
-      r_eval_with_wxyz(stop_arg_match_call, arg, values, error_arg, error_call, rlang_ns_env);
+      r_eval_with_wxyz(stop_arg_match_call,
+                       arg,
+                       values,
+                       KEEP(lazy_wrap_chr(error_arg)),
+                       KEEP(r_lazy_eval(error_call)),
+                       rlang_ns_env);
       r_stop_unreachable();
     }
   }
@@ -170,11 +179,25 @@ int arg_match(r_obj* arg,
   r_stop_unreachable();
 }
 
+int arg_match_legacy(r_obj* arg,
+                     r_obj* values,
+                     r_obj* error_arg,
+                     r_obj* error_call) {
+  struct r_lazy lazy_error_arg = { error_arg, r_null };
+  struct r_lazy lazy_error_call = { error_call, r_null };
+
+  return arg_match(arg,
+                   values,
+                   lazy_error_arg,
+                   lazy_error_call,
+                   r_lazy_null);
+}
+
 static
 int arg_match1(r_obj* arg,
                r_obj* values,
-               r_obj* error_arg,
-               r_obj* error_call) {
+               struct r_lazy error_arg,
+               struct r_lazy error_call) {
   // Simple case: one argument, we check if it's one of the values
   r_obj* const* v_values = r_chr_cbegin(values);
   int n_values = r_length(values);
@@ -185,35 +208,23 @@ int arg_match1(r_obj* arg,
     }
   }
 
-  if (error_call == r_missing_arg) {
+  r_obj* ffi_error_call = r_lazy_eval(error_call);
+  if (ffi_error_call == r_missing_arg) {
     // Replace `error_call` by environment on the stack because
     // `r_eval_with_` evaluates in an out-of-stack mask
-    error_call = r_peek_frame();
+    ffi_error_call = r_peek_frame();
   }
-  KEEP(error_call);
+  KEEP(ffi_error_call);
 
   r_eval_with_wxyz(stop_arg_match_call,
                    KEEP(wrap_chr(arg)),
                    values,
-                   KEEP(wrap_chr(error_arg)),
-                   error_call,
+                   KEEP(lazy_wrap_chr(error_arg)),
+                   ffi_error_call,
                    rlang_ns_env);
   r_stop_unreachable();
 }
 
-
-r_obj* ffi_arg_match0(r_obj* args) {
-  args = r_node_cdr(args);
-
-  r_obj* arg = r_node_car(args); args = r_node_cdr(args);
-  r_obj* values = r_node_car(args); args = r_node_cdr(args);
-  r_obj* error_arg = r_node_car(args); args = r_node_cdr(args);
-  r_obj* error_call = r_node_car(args);
-
-  int i = arg_match(arg, values, error_arg, error_call);
-  return r_str_as_character(r_chr_get(values, i));
-}
-
 static
 r_obj* wrap_chr(r_obj* arg) {
   switch (arg_match_arg_nm_type(arg)) {
@@ -228,6 +239,14 @@ r_obj* wrap_chr(r_obj* arg) {
   }
 }
 
+static
+r_obj* lazy_wrap_chr(struct r_lazy arg) {
+  r_obj* ffi_arg = KEEP(r_lazy_eval(arg));
+  r_obj* out = wrap_chr(ffi_arg);
+  FREE(1);
+  return out;
+}
+
 static
 enum r_type arg_match_arg_nm_type(r_obj* arg_nm) {
   switch (r_typeof(arg_nm)) {
@@ -244,8 +263,31 @@ enum r_type arg_match_arg_nm_type(r_obj* arg_nm) {
 }
 
 
+int cci_arg_match(r_obj* arg,
+                  r_obj* values,
+                  struct r_lazy error_arg,
+                  struct r_lazy error_call) {
+  return arg_match(arg, values, error_arg, error_call, r_lazy_null);
+}
+
+r_obj* ffi_arg_match0(r_obj* args) {
+  args = r_node_cdr(args);
+
+  r_obj* arg = r_node_car(args); args = r_node_cdr(args);
+  r_obj* values = r_node_car(args); args = r_node_cdr(args);
+  r_obj* frame = r_node_car(args);
+
+  struct r_lazy error_arg = { .x = syms.arg_nm, .env = frame };
+  struct r_lazy error_call = { .x = r_syms.error_call, .env = frame };
+  struct r_lazy call = { .x = frame, .env = r_null };
+
+  int i = arg_match(arg, values, error_arg, error_call, call);
+  return r_str_as_character(r_chr_get(values, i));
+}
+
+
 void rlang_init_arg(r_obj* ns) {
-  stop_arg_match_call = r_parse("stop_arg_match(w, x, y, z)");
+  stop_arg_match_call = r_parse("stop_arg_match(w, values = x, error_arg = y, error_call = z)");
   r_preserve(stop_arg_match_call);
 }
 
