@@ -2,7 +2,7 @@ static r_obj* auto_name_call;
 static r_obj* dots_homonyms_values;
 static r_obj* dots_ignore_empty_values;
 static r_obj* empty_spliced_arg;
-static r_obj* glue_unquote_fn;
+static r_obj* glue_embrace_fn;
 static r_obj* quosures_attrib;
 static r_obj* splice_box_attrib;
 static r_obj* abort_dots_homonyms_ns_sym;
@@ -20,3 +20,13 @@ enum dots_homonyms arg_match_homonyms(r_obj* homonyms);
 
 static
 enum arg_named arg_match_named(r_obj* named);
+
+static inline
+bool should_ignore(struct dots_capture_info* p_capture_info,
+                   r_obj* expr,
+                   r_obj* name,
+                   bool last);
+
+static inline
+void ignore(struct dots_capture_info* p_capture_info,
+            r_obj* node);
