@@ -1003,15 +1003,18 @@ test_that("can shrink and grow dynamic barrier vectors", {
   expect_equal(dyn_count(arr), 2)
 })
 
-test_that("can get and poke elements", {
+test_that("can get, push, and poke elements", {
   arr <- new_dyn_vector("logical", 3)
   dyn_push_back(arr, TRUE)
+  dyn_lgl_push_back(arr, TRUE)
+  expect_equal(dyn_lgl_get(arr, 0L), TRUE)
+  expect_equal(dyn_lgl_get(arr, 1L), TRUE)
   dyn_lgl_poke(arr, 0L, FALSE)
   expect_equal(dyn_lgl_get(arr, 0L), FALSE)
 
   arr <- new_dyn_vector("integer", 3)
   dyn_push_back(arr, 1L)
-  dyn_push_back(arr, 2L)
+  dyn_int_push_back(arr, 2L)
   expect_equal(dyn_int_get(arr, 0L), 1L)
   expect_equal(dyn_int_get(arr, 1L), 2L)
   dyn_int_poke(arr, 0L, 10L)
@@ -1019,27 +1022,44 @@ test_that("can get and poke elements", {
 
   arr <- new_dyn_vector("double", 3)
   dyn_push_back(arr, 1.5)
-  dyn_dbl_poke(arr, 0L, 2.5)
-  expect_equal(dyn_dbl_get(arr, 0L), 2.5)
+  dyn_dbl_push_back(arr, 2.5)
+  expect_equal(dyn_dbl_get(arr, 0L), 1.5)
+  expect_equal(dyn_dbl_get(arr, 1L), 2.5)
+  dyn_dbl_poke(arr, 0L, 3.5)
+  expect_equal(dyn_dbl_get(arr, 0L), 3.5)
 
   arr <- new_dyn_vector("complex", 3)
   dyn_push_back(arr, 0i)
-  dyn_cpl_poke(arr, 0L, 1i)
-  expect_equal(dyn_cpl_get(arr, 0L), 1i)
+  dyn_cpl_push_back(arr, 1i)
+  expect_equal(dyn_cpl_get(arr, 0L), 0i)
+  expect_equal(dyn_cpl_get(arr, 1L), 1i)
+  dyn_cpl_poke(arr, 0L, 2i)
+  expect_equal(dyn_cpl_get(arr, 0L), 2i)
 
   arr <- new_dyn_vector("raw", 3)
   dyn_push_back(arr, as.raw(1))
-  dyn_raw_poke(arr, 0L, as.raw(2))
-  expect_equal(dyn_raw_get(arr, 0L), as.raw(2))
+  dyn_raw_push_back(arr, as.raw(2))
+  expect_equal(dyn_raw_get(arr, 0L), as.raw(1))
+  expect_equal(dyn_raw_get(arr, 1L), as.raw(2))
+  dyn_raw_poke(arr, 0L, as.raw(3))
+  expect_equal(dyn_raw_get(arr, 0L), as.raw(3))
 
   arr <- new_dyn_vector("character", 3)
-  dyn_push_back(arr, chr_get("foo", 0L))
-  val <- chr_get("bar", 0L)
-  dyn_chr_poke(arr, 0L, val)
-  expect_true(identical(dyn_chr_get(arr, 0L), val))
+  foo <- chr_get("foo", 0L)
+  bar <- chr_get("bar", 0L)
+  dyn_push_back(arr, foo)
+  dyn_chr_push_back(arr, bar)
+  expect_true(identical(dyn_chr_get(arr, 0L), foo))
+  expect_true(identical(dyn_chr_get(arr, 1L), bar))
+  baz <- chr_get("bar", 0L)
+  dyn_chr_poke(arr, 0L, baz)
+  expect_true(identical(dyn_chr_get(arr, 0L), baz))
 
   arr <- new_dyn_vector("list", 3)
   dyn_push_back(arr, 1:2)
+  dyn_list_push_back(arr, 3:4)
+  expect_equal(dyn_list_get(arr, 0L), 1:2)
+  expect_equal(dyn_list_get(arr, 1L), 3:4)
   dyn_list_poke(arr, 0L, 11:12)
   expect_equal(dyn_list_get(arr, 0L), 11:12)
 })
